import { DOCUMENT, CommonModule } from '@angular/common';
import { NgbActiveModal, NgbModal, NgbModule } from '@ng-bootstrap/ng-bootstrap';
import { AngularDraggableModule } from 'angular2-draggable';
import { isObject } from 'rxjs/internal/util/isObject';
import { trigger, state, style, animate, transition } from '@angular/animations';
import { Component, Input, ViewEncapsulation, Renderer2, Inject, ElementRef, Injectable, NgModule, Pipe } from '@angular/core';
import { Observable } from 'rxjs';
import { share } from 'rxjs/operators';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalComponent {
    // Documentation: https://ng-bootstrap.github.io/#/components/modal
    /**
     * @param {?} document
     * @param {?} innerModaleActiveModal
     * @param {?} renderer
     * @param {?} elRef
     */
    constructor(document, innerModaleActiveModal, renderer, elRef) {
        this.innerModaleActiveModal = innerModaleActiveModal;
        this.renderer = renderer;
        this.elRef = elRef;
        this.title = 'Modal title';
        this.closeButton = true;
        this._document = document;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.renderer.addClass(this._document.body, 'gst-modal-open');
        this.renderer.addClass(this._document.body, 'noselect');
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this._document.querySelector('.modal') == null) {
            this.renderer.removeClass(this._document.body, 'gst-modal-open');
            this.renderer.removeClass(this._document.body, 'noselect');
        }
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    onMoveEnd($event) {
        /** @type {?} */
        let maxWidth = Math.max(document.body.scrollWidth, document.documentElement.scrollWidth, document.body.offsetWidth, document.documentElement.offsetWidth, document.documentElement.clientWidth);
        /** @type {?} */
        let element = document.getElementById(this.elRef.nativeElement.id + '_Modal');
        /** @type {?} */
        let x = element.offsetLeft + $event.x;
        /** @type {?} */
        let y = element.offsetTop + $event.y;
        /** @type {?} */
        let eventX = $event.x < -element.offsetLeft ? -element.offsetLeft : $event.x;
        /** @type {?} */
        let eventY = $event.y < -element.offsetTop ? -element.offsetTop : $event.y;
        if (y < 0) {
            element.style.transform = 'translate(' + eventX + 'px, -' + element.offsetTop + 'px)';
        }
        if (x < 0) {
            element.style.transform = 'translate(-' + element.offsetLeft + 'px, ' + eventY + 'px)';
        }
        if (x + element.offsetWidth > maxWidth) {
            element.style.transform = 'translate(' + (maxWidth - element.offsetWidth - element.offsetLeft) + 'px, ' + eventY + 'px)';
        }
    }
}
ModalComponent.decorators = [
    { type: Component, args: [{
                selector: 'gst-modal',
                encapsulation: ViewEncapsulation.None,
                template: `
        <div id="{{elRef.nativeElement.id}}_Modal" ngDraggable [handle]="myHandle" class="modal-outer modal-draggable onselect" (movingOffset)="onMoveEnd($event)">
            <div #myHandle class="modal-header">
                <h4 class="modal-title">{{title}}</h4>
                <div class="modal-action-bar">
                    <button *ngIf="closeButton" type="button" class="close" aria-label="Close" (click)="innerModaleActiveModal.dismiss()">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
            </div>
            <div class="modal-body">
                <ng-content></ng-content>
            </div>
        </div>
    `,
                styles: ["body.gst-modal-open,body.modal-open{overflow:hidden}@keyframes fadein{0%{opacity:0}100%{opacity:1}}@-webkit-keyframes fadein{0%{opacity:0}100%{opacity:1}}.modal-backdrop{background:rgba(0,0,0,.4);position:fixed;top:0;left:0;width:100vw;height:100vh;z-index:200;opacity:0}ngb-modal-window{position:fixed;z-index:200;top:0;left:0;width:100vw;height:100vh}.modal .modal-content{position:fixed;z-index:201;overflow:auto;background:rgba(0,0,0,.4);width:100vw;height:100vh;opacity:0;-webkit-animation:.5s forwards fadein;animation:.5s forwards fadein}.modal .modal-outer{width:50%;left:25%;position:relative;z-index:201;top:60px;background:#fff;border-radius:2px;margin-bottom:60px}.modal-header{position:relative;box-sizing:border-box;-moz-box-sizing:border-box;cursor:move;background-color:#f5f5f5;height:40px}.modal-header:before{content:'';position:absolute;width:calc(100% - 30px);left:15px;top:21px;border-top:1px solid #d9dde5}.modal-header h4{margin:0}.modal-header .modal-title{position:relative;display:inline-block;left:30px;background-color:#f5f5f5;color:#48a770;font-weight:300;padding:0 20px}.modal-header .modal-action-bar{position:absolute;right:25px;top:5px;height:100%}.modal-header .modal-action-bar button.close{cursor:pointer;border:0;width:30px;height:30px;line-height:30px;background:#f5f5f5;font-size:1.5em}.modal-header .modal-action-bar button.close:hover{color:#fff;background:#e88b81}.modal-body{position:relative;z-index:1;background:#fff}.modal-lg .modal-outer{width:70%;left:15%}.modal-sm .modal-outer{width:40%;left:30%}.modal-title{line-height:40px;font-size:18px;font-weight:300;max-width:91%;display:inline-block;text-overflow:ellipsis;white-space:nowrap;overflow:hidden;color:#444}@media only screen and (max-width:1440px){.modal .modal-outer{width:75%;left:12.5%}.modal-lg .modal-outer{width:90%;left:5%}.modal-sm .modal-outer{width:60%;left:20%}}@media only screen and (max-device-width:480px),(max-width:911px){.modal .modal-outer{width:100%;left:0;top:0;margin-bottom:0;height:100vh}.modal-header{position:fixed;top:0;width:100%;z-index:100}.modal-body{height:calc(100vh - 40px);top:40px;overflow:auto}.modal-lg .modal-outer,.modal-sm .modal-outer{width:100%;left:0;top:0}}.noselect{-webkit-touch-callout:none;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.onselect{-webkit-touch-callout:text;-webkit-user-select:text;-moz-user-select:text;-ms-user-select:text;user-select:text}"]
            }] }
];
/** @nocollapse */
ModalComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: NgbActiveModal },
    { type: Renderer2 },
    { type: ElementRef }
];
ModalComponent.propDecorators = {
    title: [{ type: Input }],
    closeButton: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {...?} args
 * @return {?}
 */
function extend(...args) {
    for (let i = 1; i < args.length; i++) {
        for (let key in args[i]) {
            if (args[i].hasOwnProperty(key)) {
                args[0][key] = args[i][key];
            }
        }
    }
    return args[0];
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class GSTModal {
    /**
     * @param {?} modalService
     */
    constructor(modalService) {
        this.modalService = modalService;
    }
    /**
     * @param {?} content
     * @param {?=} options
     * @return {?}
     */
    open(content, options) {
        /** @type {?} */
        let def = {
            backdrop: false,
            container: '.modal-outer-template-div',
            keyboard: false,
            size: '',
            windowClass: ''
        };
        options = extend({}, def, options);
        return this.modalService.open(content, options);
    }
}
GSTModal.decorators = [
    { type: Injectable }
];
/** @nocollapse */
GSTModal.ctorParameters = () => [
    { type: NgbModal }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: ModalModule,
            providers: [NgbModule, GSTModal]
        };
    }
}
ModalModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    NgbModule,
                    AngularDraggableModule
                ],
                providers: [],
                // leave empty to avoid multiple instances
                declarations: [ModalComponent],
                exports: [ModalComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Allow ngFor to iterate on object collection
 *
 * ref = {
 *  a1: "valeur,
 *  a2: "valeur,
 *  a3: "valeur,
 *  }
 *
 *  ou encore
 *
 * ref = {
 *  a1: {foo:'bar', foo1:'bar1'},
 *  a2: {foo:'bar', foo1:'bar1'},
 *  a3: {foo:'bar', foo1:'bar1'}
 *  }
 *
 * exemple
 * <select class="form__input" type="text" ngControl="situationZoneClimatique">
 * <option *ngFor="let c of references?.climat | readOnly" value="{{ c.key }}">{{ c.value }}</option>
 * </select>
 *
 */
class ReadOnlyPipe {
    /**
     * @param {?} value
     * @param {?=} args
     * @return {?}
     */
    transform(value, args = []) {
        if (value == null) {
            return null;
        }
        if (value instanceof Map) {
            return this.manageMapObject(value);
        }
        else if (Array.isArray(value) || typeof value === 'object') {
            return this.manageArray(value, args);
        }
        return value;
    }
    /**
     * @private
     * @param {?} value
     * @return {?}
     */
    manageMapObject(value) {
        this.result = [];
        value.forEach((obj, key) => {
            if (obj instanceof Object) {
                obj = this.manageObjectList(obj, 1);
                this.result = this.result.concat(obj);
            }
            else {
                this.result.push({ key, obj });
            }
        });
        return this.result;
    }
    /**
     * @private
     * @param {?} objList
     * @param {?=} depth
     * @param {?=} into
     * @return {?}
     */
    manageObjectList(objList, depth = 0, into = 0) {
        /** @type {?} */
        let keyArr = Object.keys(objList);
        /** @type {?} */
        let arrRes = {};
        keyArr.forEach((key) => {
            if (this.isCyclic(objList[key])) {
                if (into <= depth) {
                    into++;
                    arrRes[key] = this.manageObjectList(objList[key], depth, into);
                }
            }
            else {
                arrRes[key] = objList[key];
            }
        });
        return arrRes;
    }
    /**
     * @private
     * @param {?} obj
     * @return {?}
     */
    isCyclic(obj) {
        /** @type {?} */
        let seenObjects = [];
        /** @type {?} */
        let detect = (o) => {
            if (typeof o === 'object') {
                if (seenObjects.indexOf(o) !== -1) {
                    return true;
                }
                seenObjects.push(o);
                for (let key in o) {
                    if (o.hasOwnProperty(key) && detect(o[key])) {
                        return true;
                    }
                }
            }
            return false;
        };
        return detect(obj);
    }
    /**
     * @private
     * @param {?} value
     * @param {?=} args
     * @return {?}
     */
    manageArray(value, args = []) {
        /** @type {?} */
        let keyArr = Object.keys(value);
        /** @type {?} */
        let dataArr = [];
        /** @type {?} */
        let keyName = args[0] ? args[0] : 'key';
        keyArr.forEach((key) => {
            if (!isObject(value[key])) {
                value[key] = { value: value[key] };
            }
            value[key][keyName] = key;
            dataArr.push(value[key]);
        });
        if (args[1]) {
            dataArr.sort((a, b) => {
                return a[keyName] > b[keyName] ? 1 : -1;
            });
        }
        return dataArr;
    }
}
ReadOnlyPipe.decorators = [
    { type: Pipe, args: [{
                name: 'readOnly',
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class GSTPipeModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: GSTPipeModule,
            providers: [],
        };
    }
}
GSTPipeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ReadOnlyPipe
                ],
                exports: [
                    ReadOnlyPipe
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BusyService {
    constructor() {
        this.busyObservable = ((/** @type {?} */ (new Observable((observer) => {
            this.busyObserver = observer;
        })))).pipe(share());
    }
    /**
     * @return {?}
     */
    show() {
        if (this.busyObserver) {
            this.busyObserver.next(true);
        }
    }
    /**
     * @return {?}
     */
    hide() {
        if (this.busyObserver) {
            this.busyObserver.next(false);
        }
    }
}
BusyService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
BusyService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BusyComponent {
    /**
     * @param {?} busyService
     */
    constructor(busyService) {
        this.busyService = busyService;
        /**
         * Set the minimum loader display time
         */
        this.minTTL = 400;
        this.startCount = 0;
        this.firstStartTime = 0;
        this.state = 'hide';
        this.divStateHide = true;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.createServiceSubscription();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
    /**
     * @return {?}
     */
    startBusy() {
        this.state = 'show';
        this.startCount++;
        if (this.firstStartTime === 0) {
            this.firstStartTime = Date.now();
        }
    }
    /**
     * @return {?}
     */
    stopBusy() {
        this.startCount--;
        if (this.startCount <= 0) {
            /** @type {?} */
            const now = Date.now();
            if (now - this.firstStartTime >= this.minTTL) {
                this.state = 'hide';
                this.firstStartTime = 0;
            }
            else {
                setTimeout(() => {
                    this.stopBusy();
                }, this.minTTL - (now - this.firstStartTime));
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    animation(event) {
        if (event.phaseName === 'done' && event.toState === 'hide') {
            this.divStateHide = true;
        }
        if (event.phaseName === 'start' && event.toState === 'show') {
            this.divStateHide = false;
        }
    }
    /**
     * @private
     * @return {?}
     */
    createServiceSubscription() {
        this.subscription = this.busyService.busyObservable.subscribe((show) => {
            if (show) {
                this.startBusy();
            }
            else {
                this.stopBusy();
            }
        });
    }
}
BusyComponent.decorators = [
    { type: Component, args: [{
                selector: 'gst-busy',
                template: "<div class=\"ng2-busy\"\n[@getbusy]=\"state\"\n(@getbusy.start)=\"animation($event)\"\n(@getbusy.done)=\"animation($event)\"\n[style.display]=\"divStateHide ? 'none' : 'block'\"\n[style.opacity]=\"divStateHide ? 0 : 1\"\n>\n    <ng-content></ng-content>\n</div>",
                animations: [
                    trigger('getbusy', [
                        state('show', style({
                            opacity: 1
                        })),
                        state('hide', style({
                            opacity: 0
                        })),
                        transition('* => *', animate('250ms ease-in-out')),
                    ])
                ],
                styles: [".ng2-busy{display:none;opacity:0;position:fixed;top:0;left:0;width:100vw;height:100vh;text-align:center;z-index:100000;background:rgba(0,0,0,.2);transition:.5s ease-in-out}"]
            }] }
];
/** @nocollapse */
BusyComponent.ctorParameters = () => [
    { type: BusyService }
];
BusyComponent.propDecorators = {
    minTTL: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BusyModule {
}
BusyModule.decorators = [
    { type: NgModule, args: [{
                providers: [],
                // leave empty to avoid multiple instances
                imports: [BrowserAnimationsModule],
                declarations: [BusyComponent],
                exports: [BusyComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ModalModule, ModalComponent, GSTModal, GSTPipeModule, ReadOnlyPipe, BusyComponent, BusyModule, BusyService, extend };

//# sourceMappingURL=gst-components.js.map